package pl.edu.amu.wmi.min.relativecardinality.functions;

import org.apache.commons.math3.analysis.UnivariateFunction;
import org.apache.commons.math3.optim.MaxEval;
import org.apache.commons.math3.optim.nonlinear.scalar.GoalType;
import org.apache.commons.math3.optim.univariate.BrentOptimizer;
import org.apache.commons.math3.optim.univariate.MultiStartUnivariateOptimizer;
import org.apache.commons.math3.optim.univariate.SearchInterval;
import org.apache.commons.math3.optim.univariate.UnivariateObjectiveFunction;
import org.apache.commons.math3.optim.univariate.UnivariatePointValuePair;
import org.apache.commons.math3.util.FastMath;
import static pl.edu.amu.wmi.min.relativecardinality.functions.AbstractFunctions.rand;

/**
 *
 * @author bikol
 */
public abstract class AbstractFunctionsWithP extends AbstractFunctions {

    public AbstractFunctionsWithP(String name) {
        super(name);
    }

    @Override
    public double p(boolean lower, final double x, final double wl, final double wu) {
        if (compute() && wl < wu) {
            MultiStartUnivariateOptimizer opt = new MultiStartUnivariateOptimizer(new BrentOptimizer(1e-10, 1e-14), 10, rand);
            UnivariatePointValuePair p = opt.optimize(
                    new MaxEval(1000),
                    new SearchInterval(wl, wu),
                    lower ? GoalType.MINIMIZE : GoalType.MAXIMIZE,
                    new UnivariateObjectiveFunction(new UnivariateFunction() {

                        @Override
                        public double value(double w) {
                            return (g(f(x, w)) - g(f(x, wl))) / (g(w) - g(wl));
                        }
                    }));
            if (lower) {
                return FastMath.min(super.p(lower, x, wl, wu), p.getValue());
            } else {
                return FastMath.max(super.p(lower, x, wl, wu), FastMath.min(1.0, p.getValue()));
            }
        } else {
            return super.p(lower, x, wl, wu);
        }
    }

    abstract protected boolean compute();
}
